import 'dart:async';
import 'dart:io';
import 'dart:convert';
import 'package:dio/dio.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/serializer.dart';

import 'package:jira_cloud/model/search_request_bean.dart';
import 'package:jira_cloud/model/issue_picker_suggestions.dart';
import 'package:jira_cloud/model/search_results.dart';
import 'package:jira_cloud/model/issue_matches.dart';
import 'package:jira_cloud/model/issues_and_jql_queries.dart';

class IssueSearchApi {
  final Dio _dio;
  Serializers _serializers;

  IssueSearchApi(this._dio, this._serializers);

  /// Get issue picker suggestions
  ///
  /// Returns lists of issues matching a query string. Use this resource to provide auto-completion suggestions when the user is looking for an issue using a word or string.  This operation returns two lists:   *  &#x60;History Search&#x60; which includes issues from the user&#39;s history of created, edited, or viewed issues that contain the string in the &#x60;query&#x60; parameter.  *  &#x60;Current Search&#x60; which includes issues that match the JQL expression in &#x60;currentJQL&#x60; and contain the string in the &#x60;query&#x60; parameter.  This operation can be accessed anonymously.  **[Permissions](#permissions) required:** None.
  Future<Response<IssuePickerSuggestions>>
      comAtlassianJiraRestV2IssueIssueResourceGetIssuePickerResourceGet({
    String query,
    String currentJQL,
    String currentIssueKey,
    String currentProjectId,
    bool showSubTasks,
    bool showSubTaskParent,
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/rest/api/3/issue/picker";

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams[r'query'] = query;
    queryParams[r'currentJQL'] = currentJQL;
    queryParams[r'currentIssueKey'] = currentIssueKey;
    queryParams[r'currentProjectId'] = currentProjectId;
    queryParams[r'showSubTasks'] = showSubTasks;
    queryParams[r'showSubTaskParent'] = showSubTaskParent;
    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = [];

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'get'.toUpperCase(),
        headers: headerParams,
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      var serializer = _serializers.serializerForType(IssuePickerSuggestions);
      var data = _serializers.deserializeWith<IssuePickerSuggestions>(
          serializer, response.data);

      return Response<IssuePickerSuggestions>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }

  /// Check issues against JQL
  ///
  /// Checks whether one or more issues would be returned by one or more JQL queries.  **[Permissions](#permissions) required:** None, however, issues are only matched against JQL queries where the user has:   *  *Browse projects* [project permission](https://confluence.atlassian.com/x/yodKLg) for the project that the issue is in.  *  If [issue-level security](https://confluence.atlassian.com/x/J4lKLg) is configured, issue-level security permission to view the issue.
  Future<Response<IssueMatches>>
      comAtlassianJiraRestV2MatchIssueMatchResourceMatchIssuesPost(
    IssuesAndJQLQueries issuesAndJQLQueries, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/rest/api/3/jql/match";

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = ["application/json"];

    var serializedBody = _serializers.serialize(issuesAndJQLQueries);
    var jsonissuesAndJQLQueries = json.encode(serializedBody);
    bodyData = jsonissuesAndJQLQueries;

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'post'.toUpperCase(),
        headers: headerParams,
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      var serializer = _serializers.serializerForType(IssueMatches);
      var data =
          _serializers.deserializeWith<IssueMatches>(serializer, response.data);

      return Response<IssueMatches>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }

  /// Search for issues using JQL (GET)
  ///
  /// Searches for issues using [JQL](https://confluence.atlassian.com/x/egORLQ).  If the JQL query expression is too large to be encoded as a query parameter, use the [POST](#api-rest-api-3-search-post) version of this resource.  This operation can be accessed anonymously.  **[Permissions](#permissions) required:** Issues are included in the response where the user has:   *  *Browse projects* [project permission](https://confluence.atlassian.com/x/yodKLg) for the project containing the issue.  *  If [issue-level security](https://confluence.atlassian.com/x/J4lKLg) is configured, issue-level security permission to view the issue.
  Future<Response<SearchResults>>
      comAtlassianJiraRestV2SearchSearchResourceSearchForIssuesUsingJqlGet({
    String jql,
    int startAt,
    int maxResults,
    String validateQuery,
    List<String> fields,
    String expand,
    List<String> properties,
    bool fieldsByKeys,
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/rest/api/3/search";

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams[r'jql'] = jql;
    queryParams[r'startAt'] = startAt;
    queryParams[r'maxResults'] = maxResults;
    queryParams[r'validateQuery'] = validateQuery;
    queryParams[r'fields'] = fields;
    queryParams[r'expand'] = expand;
    queryParams[r'properties'] = properties;
    queryParams[r'fieldsByKeys'] = fieldsByKeys;
    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = [];

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'get'.toUpperCase(),
        headers: headerParams,
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      var serializer = _serializers.serializerForType(SearchResults);
      var data = _serializers.deserializeWith<SearchResults>(
          serializer, response.data);

      return Response<SearchResults>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }

  /// Search for issues using JQL (POST)
  ///
  /// Searches for issues using [JQL](https://confluence.atlassian.com/x/egORLQ).  There is a [GET](#api-rest-api-3-search-get) version of this resource that can be used for smaller JQL query expressions.  This operation can be accessed anonymously.  **[Permissions](#permissions) required:** Issues are included in the response where the user has:   *  *Browse projects* [project permission](https://confluence.atlassian.com/x/yodKLg) for the project containing the issue.  *  If [issue-level security](https://confluence.atlassian.com/x/J4lKLg) is configured, issue-level security permission to view the issue.
  Future<Response<SearchResults>>
      comAtlassianJiraRestV2SearchSearchResourceSearchForIssuesUsingJqlPost(
    SearchRequestBean searchRequestBean, {
    CancelToken cancelToken,
    Map<String, String> headers,
  }) async {
    String _path = "/rest/api/3/search";

    Map<String, dynamic> queryParams = {};
    Map<String, String> headerParams = Map.from(headers ?? {});
    dynamic bodyData;

    queryParams.removeWhere((key, value) => value == null);
    headerParams.removeWhere((key, value) => value == null);

    List<String> contentTypes = ["application/json"];

    var serializedBody = _serializers.serialize(searchRequestBean);
    var jsonsearchRequestBean = json.encode(serializedBody);
    bodyData = jsonsearchRequestBean;

    return _dio
        .request(
      _path,
      queryParameters: queryParams,
      data: bodyData,
      options: Options(
        method: 'post'.toUpperCase(),
        headers: headerParams,
        contentType:
            contentTypes.isNotEmpty ? contentTypes[0] : "application/json",
      ),
      cancelToken: cancelToken,
    )
        .then((response) {
      var serializer = _serializers.serializerForType(SearchResults);
      var data = _serializers.deserializeWith<SearchResults>(
          serializer, response.data);

      return Response<SearchResults>(
        data: data,
        headers: response.headers,
        request: response.request,
        redirects: response.redirects,
        statusCode: response.statusCode,
        statusMessage: response.statusMessage,
        extra: response.extra,
      );
    });
  }
}
